@@ -23,7 +23,7 @@ from functions_framework.background_event import BackgroundEvent
 from functions_framework.exceptions import EventConversionException
 from google.cloud.functions.context import Context
 
-_CLOUDEVENT_SPEC_VERSION = "1.0"
+_CLOUD_EVENT_SPEC_VERSION = "1.0"
 
 # Maps background/legacy event types to their equivalent CloudEvent types.
 # For more info on event mappings see
@@ -42,10 +42,10 @@ _BACKGROUND_TO_CE_TYPE = {
     "providers/firebase.auth/eventTypes/user.create": "google.firebase.auth.user.v1.created",
     "providers/firebase.auth/eventTypes/user.delete": "google.firebase.auth.user.v1.deleted",
     "providers/google.firebase.analytics/eventTypes/event.log": "google.firebase.analytics.log.v1.written",
-    "providers/google.firebase.database/eventTypes/ref.create": "google.firebase.database.document.v1.created",
-    "providers/google.firebase.database/eventTypes/ref.write": "google.firebase.database.document.v1.written",
-    "providers/google.firebase.database/eventTypes/ref.update": "google.firebase.database.document.v1.updated",
-    "providers/google.firebase.database/eventTypes/ref.delete": "google.firebase.database.document.v1.deleted",
+    "providers/google.firebase.database/eventTypes/ref.create": "google.firebase.database.ref.v1.created",
+    "providers/google.firebase.database/eventTypes/ref.write": "google.firebase.database.ref.v1.written",
+    "providers/google.firebase.database/eventTypes/ref.update": "google.firebase.database.ref.v1.updated",
+    "providers/google.firebase.database/eventTypes/ref.delete": "google.firebase.database.ref.v1.deleted",
     "providers/cloud.storage/eventTypes/object.change": "google.cloud.storage.object.v1.finalized",
 }
 
@@ -94,7 +94,7 @@ _SERVICE_BACKGROUND_TO_CE = {
 # for the subject.
 _CE_SERVICE_TO_RESOURCE_RE = {
     _FIREBASE_CE_SERVICE: re.compile(r"^(projects/[^/]+)/(events/[^/]+)$"),
-    _FIREBASE_DB_CE_SERVICE: re.compile(r"^(projects/[^/]/instances/[^/]+)/(refs/.+)$"),
+    _FIREBASE_DB_CE_SERVICE: re.compile(r"^projects/_/(instances/[^/]+)/(refs/.+)$"),
     _FIRESTORE_CE_SERVICE: re.compile(
         r"^(projects/[^/]+/databases/\(default\))/(documents/.+)$"
     ),
@@ -114,7 +114,7 @@ _FIREBASE_AUTH_METADATA_FIELDS_CE_TO_BACKGROUND = {
 }
 
 
-def background_event_to_cloudevent(request) -> CloudEvent:
+def background_event_to_cloud_event(request) -> CloudEvent:
     """Converts a background event represented by the given HTTP request into a CloudEvent."""
     event_data = marshal_background_event_data(request)
     if not event_data:
@@ -131,9 +131,14 @@ def background_event_to_cloudevent(request) -> CloudEvent:
     new_type = _BACKGROUND_TO_CE_TYPE[context.event_type]
 
     service, resource, subject = _split_resource(context)
+    source = f"//{service}/{resource}"
 
     # Handle Pub/Sub events.
     if service == _PUBSUB_CE_SERVICE:
+        if "messageId" not in data:
+            data["messageId"] = context.event_id
+        if "publishTime" not in data:
+            data["publishTime"] = context.timestamp
         data = {"message": data}
 
     # Handle Firebase Auth events.
@@ -147,13 +152,30 @@ def background_event_to_cloudevent(request) -> CloudEvent:
             uid = data["uid"]
             subject = f"users/{uid}"
 
+    # Handle Firebase DB events.
+    if service == _FIREBASE_DB_CE_SERVICE:
+        # The CE source of firebasedatabase CloudEvents includes location information
+        # that is inferred from the 'domain' field of legacy events.
+        if "domain" not in event_data:
+            raise EventConversionException(
+                "Invalid FirebaseDB event payload: missing 'domain'"
+            )
+
+        domain = event_data["domain"]
+        location = "us-central1"
+        if domain != "firebaseio.com":
+            location = domain.split(".")[0]
+
+        resource = f"projects/_/locations/{location}/{resource}"
+        source = f"//{service}/{resource}"
+
     metadata = {
         "id": context.event_id,
         "time": context.timestamp,
-        "specversion": _CLOUDEVENT_SPEC_VERSION,
+        "specversion": _CLOUD_EVENT_SPEC_VERSION,
         "datacontenttype": "application/json",
         "type": new_type,
-        "source": f"//{service}/{resource}",
+        "source": source,
     }
 
     if subject:
@@ -162,7 +184,7 @@ def background_event_to_cloudevent(request) -> CloudEvent:
     return CloudEvent(metadata, data)
 
 
-def is_convertable_cloudevent(request) -> bool:
+def is_convertable_cloud_event(request) -> bool:
     """Is the given request a known CloudEvent that can be converted to background event."""
     if is_binary(request.headers):
         event_type = request.headers.get("ce-type")
@@ -185,7 +207,7 @@ def _split_ce_source(source) -> Tuple[str, str]:
     return match.group(1), match.group(2)
 
 
-def cloudevent_to_background_event(request) -> Tuple[Any, Context]:
+def cloud_event_to_background_event(request) -> Tuple[Any, Context]:
     """Converts a background event represented by the given HTTP request into a CloudEvent."""
     try:
         event = from_http(request.headers, request.get_data())
@@ -201,6 +223,10 @@ def cloudevent_to_background_event(request) -> Tuple[Any, Context]:
             resource = {"service": service, "name": name, "type": _PUBSUB_MESSAGE_TYPE}
             if "message" in data:
                 data = data["message"]
+            if "messageId" in data:
+                del data["messageId"]
+            if "publishTime" in data:
+                del data["publishTime"]
         elif service == _FIREBASE_AUTH_CE_SERVICE:
             resource = name
             if "metadata" in data:
@@ -214,6 +240,9 @@ def cloudevent_to_background_event(request) -> Tuple[Any, Context]:
                 "service": service,
                 "type": data["kind"],
             }
+        elif service == _FIREBASE_DB_CE_SERVICE:
+            name = re.sub("/locations/[^/]+", "", name)
+            resource = f"{name}/{event['subject']}"
         else:
             resource = f"{name}/{event['subject']}"
 
@@ -288,7 +317,7 @@ def marshal_background_event_data(request):
             "data": {
                 "@type": _PUBSUB_MESSAGE_TYPE,
                 "data": request_data["message"]["data"],
-                "attributes": request_data["message"]["attributes"],
+                "attributes": request_data["message"].get("attributes", {}),
             },
         }
     except (AttributeError, KeyError, TypeError):
